#include "AplicacaoBase.hpp"

#if OGRE_PLATFORM == OGRE_PLATFORM_APPLE
#include <CoreFoundation/CoreFoundation.h>

// Retorna o caminho da aplicacao no MAC OS, para localizar os arquivos de 
// configuracao
std::string macBundlePath()
{
   char path[1024];
   CFBundleRef mainBundle = CFBundleGetMainBundle();
   assert(mainBundle);

   CFURLRef mainBundleURL = CFBundleCopyBundleURL(mainBundle);
   assert(mainBundleURL);

   CFStringRef cfStringRef = CFURLCopyFileSystemPath( mainBundleURL, 
         kCFURLPOSIXPathStyle);
   assert(cfStringRef);

   CFStringGetCString(cfStringRef, path, 1024, kCFStringEncodingASCII);

   CFRelease(mainBundleURL);
   CFRelease(cfStringRef);

   return std::string(path);
}
#endif

// Construtor
AplicacaoBase::AplicacaoBase()
{
   // Localiza o caminho dos arquivos de configuracao a partir do diretorio 
   // atual, ou, no MAC OS, atraves do caminho completo
#if OGRE_PLATFORM == OGRE_PLATFORM_APPLE
   caminho = macBundlePath() + "/Contents/Resources/";
#else
   caminho = "";
#endif
}

// Destrutor
AplicacaoBase::~AplicacaoBase()
{
   // Remove o frame listener
   if (frameListener)
      delete frameListener;
   
   // Remove o root
   if (root)
      delete root;
}

// Inicia a aplicacao
void AplicacaoBase::inicia()
{
   // Cria os elementos da cena e o frame listener
   criaCamera();
   criaViewports();
   criaLuz();
   criaCena();
   criaFrameListener();

   // Inicia o loop de renderizacao da cena
   root->startRendering();

   // Ao final, o proprio Ogre se encarrega de destruir os objetos criados
}

// Configura a aplicacao
bool AplicacaoBase::configura()
{
   // Localiza o arquivo de plugins
   String caminhoPlugins;
#ifndef OGRE_STATIC_LIB
   caminhoPlugins = caminho + "../config/plugins.cfg";
#endif
   
   // Cria um log manager manualmente para o root, neste caso, desabilitando
   // mensagens de log impressas no stdout
   LogManager* lm = new LogManager();
   lm->createLog(caminho + "Ogre.log", true, false, false);

   // Cria o root
   root = new Root(caminhoPlugins, caminho + "../config/window.cfg", "");
   
   // Configura os recursos
   configuraRecursos();
   carregaRecursos();

   // Interrompe se houver erro na configuracao da janela
   if (!configuraJanela())
      return false;

   // Cria o gerenciador de cena
   sceneMgr = root->createSceneManager(ST_GENERIC, "ExampleSMInstance");
   
   return true;
}

// Configura a janela, definindo as propriedades de tamanho, modo fullscreen e
// sistema de renderizacao presentes no arquivo window.cfg
bool AplicacaoBase::configuraJanela()
{
   // Se nao for possivel carregar o arquivo, abre uma janela pedindo essas
   // informacoes do usuario
   if (!root->restoreConfig())
   {
      // Se o usuario cancelar a janela, ou se houver erro
      if (!root->showConfigDialog())
         return false;
   }
   // Cria a janela de acordo com as configuracoes dadas
   janela = root->initialise(true, "Avator");

   // Se houver camera, reposiciona a janela da aplicacao mais para baixo, para
   // dar espaco para a janela da camera
   if (usaCamera)
      janela->reposition((TELA_RES_X - TELA_X)/2, (TELA_RES_Y - TELA_Y)/2 
            + 105);
   return true;
}

// Cria a viewport do programa
void AplicacaoBase::criaViewports()
{
   // Cria apenas uma viewport (nao divide a janela)
   Viewport* vp = janela->addViewport(camera);

   // Cor de fundo preta
   vp->setBackgroundColour(ColourValue(0, 0, 0));

   // Ajusta a camera de acordo com as dimensoes da viewport
   Real altura = Real(vp->getActualHeight());
   Real largura = Real(vp->getActualWidth());
   camera->setAspectRatio(largura/altura);
}

// Localiza os diretorios em que estao os recursos do programa, especificados
// nos arquivos de configuracao
void AplicacaoBase::configuraRecursos()
{
   // Carrega arquivos de recursos
   ConfigFile cf;
   cf.load(caminho + "../config/resources.cfg");

   // Percorre o arquivo
   ConfigFile::SectionIterator seci = cf.getSectionIterator();

   String secName, typeName, archName;
   while (seci.hasMoreElements())
   {
      secName = seci.peekNextKey();
      ConfigFile::SettingsMultiMap* settings = seci.getNext();
      ConfigFile::SettingsMultiMap::iterator i;
      for (i = settings->begin(); i != settings->end(); ++i)
      {
         typeName = i->first;
         archName = i->second;
#if OGRE_PLATFORM == OGRE_PLATFORM_APPLE
         // Caminho no MAC OS
         ResourceGroupManager::getSingleton().addResourceLocation(
               String(macBundlePath() + "/" + archName), typeName, secName);
#else
         ResourceGroupManager::getSingleton().addResourceLocation(
               archName, typeName, secName);
#endif
      }
   }
}

// Inicializa os recursos
void AplicacaoBase::carregaRecursos()
{
   ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
}

